'-----------------------------------------------------------------------------------------

'name                     : IR-Bumper with 455kHz-modulated IR-LEDs

'copyright                : (c) 2007, Ralf Schmiedel, www.silbergold.de

'purpose                  : Input is sent as 8N1 UART-Object, then received by
'                           the Hardware-UART. The result is sent via RS-232

'micro                    : ATtiny2313-20PU

'commercial addon needed  : no

'history                  : V1.01 gibt Daten auf RS232 fr die Bildschirmanzeige aus
'                           Quarzfrequenz 18,432mhz , andere Frequenzen zwischen
'                           14,56Mhz und 20Mhz sind mglich (siehe im Programm)
'                           I2C-Adresse wird gelesen
'                           weitere I2C-Bearbeitung fehlt noch
'                           Version getestet
'                           Die transistorlose Hardware-Version funktioniert auch
'                           noch bei 3,6V
'
'                           V1.02
'                           Einlesen ber RS-232 bevor die Messung beginnt
'                           Ziel: PWMa und PWMb setzen
'                                 Version mitteilen
'                                 get Version
'                                 Baudrate ndern ?
'                                 Messung starten
'                                 Messung mit bestimmter PWM starten
'                                 Modus: 1,2,4 Messungen oder / und verknpft
'                                 Free running, till event

'                           V1.12: zustzliche Funktionen:
'                                 Progressive Scan - Scannt 1x Mit Ansteigender Pwm
'                                 Radar , Freilaufend Progressive Scans ,
'                                 Ausgabe Von Richtung Und Pwm -wert
'                                 Beim Booten Kann Die Startfunktion gewhlt
'                                 werden, indem man zuvor ins EEPROM das
'                                 entsprechende Command gespeichert hat
'                           V1.14 funktioniert
'                           V1.15 runde Sache 3.12.2007 jedoch invertierte Ausgabe
'                                 RadarScan erfolgt mit ca. 18-19scans/s
'                           V1.16 Ausgabe richtig herum, mit Hilfe von Select Code reduziert
'                           V1.17 Mode_r.2 erlaubt doppelte scangeschwindigkeit
'                                 in der progressiv Scan und radar-Routine
'                                 Optimierung des PWM-Timings ("Dead beat response")
'                                 jetzt werden max. 48 respektive 86 scans/s erreicht
'                           V1,18 PWM-Startwert for progressiv scan and radar now 15 instead 0
'                                 so now radar is 105Hz instead of 85Hz
'-----------------------------------------------------------------------------------------
'

$regfile = "ATtiny2313.dat"             ' specify the used micro

$crystal = 18432000                     '14745600 optimal would be 14560000

$baud = 38400
'for 19200 use 23 instead of 11 in AL-registers in asm-routines

$hwstack = 32                           ' default use 32 for the hardware stack

$swstack = 10                           ' default use 10 for the SW stack

$framesize = 40                         ' default use 40 for the frame space

$lib "mcsbyte.lbx"

'$eepromhex    'stores EEPROM-file as intel hex file, instead .bin


'Die Assemblerroutinen LED und NO_LED sind fr eine Taktfrequenz von 14,56MHZ ausgelegt.
'Hierbei werden genau 32 Takte zur Erzeugung der 455kHz bentigt.
'andere Quarzfrequenzen sind aber auch mglich:

' 14,7456MHz / 32 = 455kHz
' 16 MHz     / 35 = 457kHz --> 3 NOPs
' 18,2 MHz   / 40 = 455kHz --> 8 NOPs
' 18,432MHz  / 40 = 460kHz --> 8 NOPs
' 18,432MHz  / 41 = 450kHz --> 8 NOPs
' 20 MHz     / 44 = 460kHz --> 11 NOPs (eigentlich 12 aber 11 funktioniert)

'Einfach Die Entsprechende Anzahl Nops In Wait_15 Und Wait_11 Einfgen
'bei 20MHz davon 2 NOPs in LED_13takte und NO_LED_13takte


'gemessene Scan-Raten abhngig von W (wait ms) und N (Anzahl der Scans)
' W    N    L   O  mode_r             Scans/s     Dauer eines Scans:
' 1    1    1   0   48 (bin_ascii)     250         4.0ms
' 1    1    1   1   49 (hex)           417         2.4ms
' 1    1    1   2   50 (dezimal)       380         2.6ms
' 1    1    1   3   51 (binr)         468         2.1ms +
' 2    1    1   3   51                 303         3,3ms
' 4    1    1   3   51                 192         5,2ms
' 8    1    1   3   51                 111         9,1ms
'16    1    1   3   51                  62        16,1ms
'32    1    1   3   51                  34        29,4ms
'64    1    1   3   51                  17        58,8ms
'128   1    1   3   51                   8       125,0ms
'255   1    1   3   51                   4       250,0ms
' 1    3    1   3  115                 277         3,6ms
' 1    7    1   3  243                 145         6,9ms
' Radar-Mode:
'1     1    1   3   51                  48        20,8ms    (ab v1.18 62Hz)
'1     1    1   3   55   2xincr         86        11,6ms    (ab v1.18 105Hz oder 9,5ms)
'1     2    1   3   83                  28        35,7ms
'1     3    1   3  115                  20        50,0ms
'1     3    1   3  119   2xincr         36        27,8ms
'1     7    1   3  243                   8 (?)   125,0ms (ca.)
'100   3    1   3  115                   7
'255   3    1   3  115                   ca. 2/s



Config Com1 = Dummy , Synchrone = 0 , Parity = None , Stopbits = 1 , Databits = 8 , Clockpol = 0

Config Timer1 = Pwm , Pwm = 8 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down , Prescale = 1


Dim Output_format As Byte
Dim A As Byte
Dim S As Byte
Dim I As Byte                           ' Laufvariable
Dim J As Byte                           ' Universalvariable u.a.aber auch Speicher fr Ausstiegsbedingung
Dim Pwma As Byte                        ' IR-LEDs current
Dim Pwmb As Byte                        'Flash_current for Start-Bit-LED(s)
Dim Pwm As Byte                         'Buffer for PWMa value in progressiv scan routine
Dim Number_of_scans As Byte             'Number of Scans (per Scan) kind of oversampling
Dim Wms As Byte                         'Wait milliseconds between single scans in Radar- and free running mode
Dim Command As Byte                     'Stores the command to be executed
Dim Value As Byte                       'Buffer for funktion parameters
Dim Mode_r As Byte                      'Mode_Register see below


'contains the following data:
'Bits 0,1: Outputformat
'Bit  4: IRLED on / off 1 = on; 0 = off
'Bits 5,6,7: Number of Scans


'===============================================================================
'Initialisierung der Ports und Variablen


Ddrb = &B01111111                       'Alle Portb Pins sind Output
Ddrd = &B01111110                       'Nur Portd.0 ist Input (RxD Pin)
Portb = &B0100000                       'Alle Ports auf Low, Portb.6 schaltet die roten LEDs aus
Portd = &B0000000

Readeeprom Command , 1
Readeeprom Mode_r , 2
Readeeprom Pwma , 3
Readeeprom Pwmb , 4
Readeeprom Wms , 5
Gosub Read_mode


'===============================================================================
'main loop

'in the first run, the command from EEPROM will be executed
'If you store a zero (or unused command), the IR-radar will start with a prompt


Do
  If Command >= 97 Then Command = Command - 32       'lower case to upper case
  Select Case Command
  Case 65 : Gosub Set_pwm_a             'A
  Case 66 : Gosub Set_pwm_b             'B
  Case 70 : Gosub Free_running          'F
  Case 82 : Gosub Radar                 'R
  Case 77 : Gosub Set_mode              'M
  Case 87 : Gosub Set_waitms            'W
  Case 63 : Gosub Print_version         '?
  Case 83 : Gosub Scan_once             'S
  Case 80 : Gosub Progressiv_scan       'P
  Case 73 : Gosub Set_initial_command   'I
  End Select
  Print Chr(13) ; Chr(10) ; Chr(62);    '<CR><LF><Prompt>
  Command = Waitkey()
Loop
'*******************************************************************************


Set_mode:
Value = Mode_r
Gosub Get_value
Writeeeprom Value , 2
Mode_r = Value
Gosub Read_mode
Return

Read_mode:
Number_of_scans = Mode_r
Shift Number_of_scans , Right , 5
Output_format = Mode_r
Output_format = Output_format And &B00000011
Return


Set_pwm_a:                              'sets PWMa which is the ir-scan-leds pwm
Value = Pwma
Gosub Get_value
Writeeeprom Value , 3
Pwma = Value
Return

Set_pwm_b:                              'pwma Is The "flash-leds-pwm"
Value = Pwmb
Gosub Get_value
Writeeeprom Value , 4
Pwmb = Value
Return


Set_initial_command:                    'sets first activity after start up
Readeeprom Value , 1                    'an unused ascii value would show the prompt
Gosub Get_value                         'input is decimal of ascii code
Writeeeprom Value , 1
Return


Set_waitms:                             'sets waitstates in ms from 1 to 255
Value = Wms
Gosub Get_value
Writeeeprom Value , 5
Wms = Value
Return


Print_version:                          'prints version, copy right, help to rs-232
I = 6
Do
Readeeprom Value , I
Print Chr(value) ;
Incr I
Loop Until I = 127
Return



'===============================================================================

Get_value:
Print Chr(command) ; Chr(58) ; Value ; Chr(32);
Input I
If I > 0 Then Value = I
Return

'===============================================================================

Progressiv_scan:
'scans with increasing intesitiv until the first signal is received
'then outputs the angle a single <space> and then the PWM-value
' repeating this until <ESC> occures
'this Function is usefull to explore unknown terrain
'it's self running

  Pwm = 15
  Gosub Pwm_on
  Do
    Incr Pwm
    If Mode_r.2 = 1 Then Incr Pwm       'neu ab V1.17
    Pwm1a = Pwm
    Waitus 50                           'important
    Gosub Scan                          'perform scan
    If Pwm >= Pwma Then                 'max. intensitiy reached
      Pwm = 0
      Value = 0                         'nothing detected, cause PWM reached pwma,value 0 is a trick
    End If
  Loop Until Value < 255
  Gosub Pwm_off
  If Value = 0 Then Value = 255         'trick to still use the 255 for no reception
  A = Not Value
  Gosub Rs232_out
  Gosub Led_out
  Value = Pwm
  If Value = 0 Then Value = Pwma        'Pwm had reached pwma and was set to 0, here we restore it
  A = Value
  Print Chr(32);
  Gosub Rs232_out
  Print Chr(13);
Return

Radar:
'Radar mode, performs continous scans with increasing PWM, till something is
'detected - or not
'Output: 2 values; angle and pwm
'Frequency is determined by wms, number_of_scans, LED on/off
Print
Do
 Gosub Progressiv_scan
Loop Until J = 27
Return


Free_running:
'free running scans,
'Output: 1 value; angle (at a given PWMa)
'frequency depends on Number_of_scans, WMS, LED on/off
Print
Do
 Gosub Scan_once
Loop Until J = 27
Return


Scan_once:                              'performs a single scan
Pwm = Pwma
Gosub Pwm_on
Gosub Send18432
J = Inkey()
Gosub Scan
Gosub Pwm_off
Gosub Rs232_out
Print Chr(13);
Gosub Led_out
Return
End

'===============================================================================
' Hilfsfunktionen, die Code sparen und von den Scan-Funktionen verwendet werden

Scan:
Value = 0
For I = 1 To Number_of_scans
  Gosub Send18432                       'Assemblerroutine aufrufen
  S = Inkey()
  Value = S Or Value
Next I
Return

Pwm_on:
  Pwm1a = 255                           'first send overshoot to access kind of dead beat response
  Pwm1b = Pwmb                          'pwmb kann sofort gesetzt werden
  Waitus 100                            'time to preload C
  Pwm1a = Pwm                           'final pwm-value
  Waitus 200                            'wait till pwm is set
Return

Pwm_off:
  Pwm1a = 0
  Pwm1b = 0
  Waitus 250
Return

'===============================================================================
' Output functions
'
' 1. LED     can be disabled by setting mode_r.4 bit
' 2. RS-232
' 3. I2C here not used
' 4. PWM here not used

Led_out:
If Mode_r.4 > 0 Then                    'If Mode_r.4 = 1 then LED-output enable
  Reset Portb.6
  A = Not Value
  Portb.1 = A.0
  Portb.0 = A.1
  Portd.6 = A.2
  Portd.5 = A.3
  Portd.4 = A.4
  Portd.3 = A.5
  Portd.2 = A.6

End If
J = 0                                   'auch die gemeinsame Kathode ausschalten
For I = 1 To Wms
  Waitus 950
  J = J Or Inkey()
Next I
Portd = Portd And &B10000011            'alle LED - Ports wieder ausschalten
Portb = Portb And &B11111100
Set Portb.6
Return


Rs232_out:
Select Case Output_format
Case 0 : Print Bin(a) ;                 'binrer Wert in ASCII
Case 1 : Print Hex(a) ;                 'hex in ASCII
Case 2 : Print A ; Chr(32) ; Chr(32);   'Dezimal in ASCII
Case 3 : Print Chr(a) ;                 'Byte = binrer Wert
End Select
Return


'===============================================================================
' Assemblerteil
' Ein UART-Objekt mit 38,4kBit/s und einer Modulationsfrequenz von 455kHz senden

Send18432:

$asm
'*******************************************************************************
' LED leuchtet je Bit mit 12 Perioden 455kHz mit 1/ca. 8 Einschaltdauer (12,5%)


.def Al = R24
.def _0 = R25
LDI _0, 0

' UART-Objekt senden

rcall LED9                              'Startbit
rcall led1
rcall LED2
rcall LED3
rcall LED4
rcall LED5
rcall LED6
rcall LED7
rcall NO_LED                            '8.Bit nicht genutzt
rcall NO_LED
rcall NO_LED                            ' zur Sicherheit noch 1 Idle-Bit
Rjmp Ende

'===============================================================================
' Die nachfolgenden Routinen bewirken das Senden eines Bits mit der jeweiligen LED
' Eine 11 im AL-Register bewirkt das Timing fr 38,4kBd
' fr 19,2kBd eine 23 eintragen
' fr 9600Baud wre eine 47 erforderlich (nicht getestet)


Led9:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_91:
 rcall LED9_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_91
 nop
 rcall LED9_13Takte
 rcall wait_11
RET

Led1:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_11:
 rcall LED1_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_11
 nop
 rcall LED1_13Takte
 rcall wait_11
RET                                     ' 4 Takte

Led2:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_21:
 rcall LED2_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_21
 nop
 rcall LED2_13Takte
 rcall wait_11
RET

Led3:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_31:
 rcall LED3_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_31
 nop
 rcall LED3_13Takte
 rcall wait_11
RET

Led4:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_41:
 rcall LED4_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_41
 nop
 rcall LED4_13Takte
 rcall wait_11
RET

Led5:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_51:
 rcall LED5_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_51
 nop
 rcall LED5_13Takte
 rcall wait_11
RET

Led6:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_61:
 rcall LED6_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_61
 nop
 rcall LED6_13Takte
 rcall wait_11
RET

Led7:
 LDI  AL, 11                            ' 4 Takte incl. rcall
Led_71:
 rcall LED7_13Takte
 rcall wait_15
 dec AL                                 ' 4 Takte
 cp  AL, _0
 brne LED_71
 nop
 rcall LED7_13Takte
 rcall wait_11
RET


No_led:

 LDI  AL, 11                            ' 4 Takte incl. rcall
No_led_1:
 rcall NO_LED_13Takte
 rcall wait_15
 dec AL                                 '  4 Takte
 cp  AL, _0
 brne NO_LED_1
 nop
 rcall NO_LED_13Takte
 rcall wait_11
RET                                     ' 4 Takte

'===============================================================================
'Die nachfolgenden Routinen schalten die LED fr jeweils 4 Takte an
'Bei 20MHz muss hier noch ein NOP rein, damit die LEDs lang genug an bleiben (5 Takte)

Led9_13takte:
SBI  PortB, 2                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortB, 2                           ' 4 Takte
CBI  PortB, 2                           ' IR-LED1 aus fr 28 Take
ret


Led1_13takte:
SBI  PortB, 1                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortB, 1                           ' 4 Takte
CBI  PortB, 1                           ' IR-LED1 aus fr 28 Take
ret


Led2_13takte:
SBI  PortB, 0                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortB, 0                           ' 4 Takte
CBI  PortB, 0                           ' IR-LED1 aus fr 28 Take
ret

Led3_13takte:
SBI  PortD, 6                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortD, 6                           ' 4 Takte
CBI  PortD, 6                           ' IR-LED1 aus fr 28 Take
ret


Led4_13takte:
SBI  PortD, 5                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortD, 5                           ' 4 Takte
CBI  PortD, 5                           ' IR-LED1 aus fr 28 Take
ret


Led5_13takte:
SBI  PortD, 4                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortD, 4                           ' 4 Takte
CBI  PortD, 4                           ' IR-LED1 aus fr 28 Take

ret

Led6_13takte:
SBI  PortD, 3                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortD, 3                           ' 4 Takte
CBI  PortD, 3                           ' IR-LED1 aus fr 28 Take
ret

Led7_13takte:
SBI  PortD, 2                           ' IR-LED1 an fr 4 Takte (SBI und CBI bentigen 4 Takte)
' Hier hinter noch 2 NOPS fr 20MHz
SBI  PortD, 2                           ' 4 Takte
CBI  PortD, 2                           ' IR-LED1 aus fr 28 Take
ret


No_led_13takte:
nop
nop
nop
nop
nop
nop
' Hier hinter noch 2 NOPs fr 20MHz
ret

'===============================================================================
' verschiedene Warteschleifen fr die Offzeiten beim Senden
' Die Wait-Angaben gelten fr 14,56MHz Quarzfrequenz
' Je nach hherer Frequenz mssen unterschiedlich viele Nops aktiviert werden



Wait_11:                                'wartet 11 Takte incl. rcall und ret
nop
nop
nop
nop
' Bis hier NOPs fr 14,56MHz
nop
nop
nop
' Bis hier NOPs fr 16MHz
nop
nop
nop
nop
nop
' Bis hier NOPs fr 18,2MHz
nop
'Bis hier NOPs fr 20MHz zustzlich je 2 NOPs in NO-LED_13takte und LED_13takte-
'das ist erforderlich, damit die Pulsweite nicht zu schmal wird
ret

Wait_15:
rcall wait_11
ret

Ende:
$end Asm
Return
                                                 '
End